;================================================================
;
;  IR AUDIO SWITCH
;  version 1.0
;  
;  Microchip MPLAB IDE compiler   /www.microchip.com/
;
;================================================================

#include <P12F675.inc>

BG0_OFF EQU 0x2FFF
BG1_OFF EQU 0x1FFF

; __CONFIG BG1_OFF &  _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT
 __CONFIG _CPD_OFF & _CP_OFF & _BODEN_OFF & _MCLRE_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT


 cblock 0x20
  rxbuf: .10, rxchar, rxbit, txchar, txbit
  irbuf: .10, irvalue, irlength, irbit, ircnt, irtimer, irtmp
  tmr0, asw, TMPD, CTMP, DTMP1, DTMP2, DTMP3,
  areg, breg, ereg, dreg
 endc


TMPW      EQU 0x50        ; push/pop W on interrupt
TMPS      EQU 0x51        ; push/pop STATUS
TMPFSR    EQU 0x52        ; push/pop FSR

flags     EQU 0x5E

inputA    EQU 0x00        ; GP0
inputB    EQU 0x01        ; GP1
IRinput   EQU 0x02        ; GP2/INT
RXLine    EQU 0x03        ; GP3
SWInput   EQU 0x03        ; GP3
inputC    EQU 0x04        ; GP4
TXLine    EQU 0x05        ; GP5

IRSilence EQU 0x00        ; bit 0
IRDetect  EQU 0x01        ; bit 1
IRAvail   EQU 0x02        ; bit 2
rxlength  EQU .3
tmr0cnt   EQU .8

; **************    9600/ 104us    **************
delbit1   EQU .30         ;  33=104, 32=101, 31=97,6
delbit2   EQU .41         ;  49=151, 48=147, 47=145

cmp macro var1,const1
     movfw var1
     sublw const1
 endm


     ORG 0x2100           ; eeprom data
 
 de 0x01,0x06,0x00,0x00,0x00,0x00,0x00,0x00 ; power on input, ir value,
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 1
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 2
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 3
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 4
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 5
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 6
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 7
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; input 8
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; arrow up
 de 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 ; arrow down


;================================================================
;     ORG   0x3FF          ; OSCCAL
;     retlw 0x5C           ; device depended value
;================================================================

     ORG   0x0
     goto  Start

     ORG   0x4            ; interrupt vector
Interrupt                 ; save context
     movwf TMPW
     swapf STATUS,W
     bcf   STATUS,RP0     ; bank 0
     movwf TMPS
     movfw FSR
     movwf TMPFSR
     btfsc INTCON,GPIF    ; PCINT interrupt
     goto  DOPCInt
     btfsc INTCON,INTF    ; GP2/INT interrupt
     goto  DoINT0
     btfsc INTCON,T0IF    ; timer0
     goto  DoTimer0
     btfsc PIR1,TMR1IF    ; timer1
     goto  DoTimer1

IEEnd                     ; restore context
     movfw TMPFSR
     movwf FSR
     swapf TMPS,W
     movwf STATUS
     swapf TMPW,F
     swapf TMPW,W
     retfie


DoTimer0
     bcf   INTCON,T0IF
     goto  IEEnd

DoTimer1
     movfw tmr0       
     btfsc STATUS,Z		
	 goto  DoTimer1Cont
     decf  tmr0,F
     goto  DoTimer1Ex
DoTimer1Cont
     movfw rxbuf       
     btfsc STATUS,Z		
	 goto  DoTimer1Ex
     clrf  rxbuf
DoTimer1Ex
     bsf   flags,IRSilence
     bcf   PIR1,TMR1IF
     goto  IEEnd

DOPCInt
     movlw tmr0cnt
     movwf tmr0  
     call  USARTReceive
     bcf   INTCON,GPIF
     goto  IEEnd

DoINT0                    
     call  ReadTimer1
     bcf   INTCON,INTF
     goto  IEEnd



ReadTimer1
     bcf   T1CON,TMR1ON
     movfw TMR1H
     movwf irtimer
     clrf TMR1H
     clrf TMR1L
     bsf   T1CON,TMR1ON

     btfsc flags,IRDetect
     return
     btfss flags,IRSilence
     goto  ReadTimer10
     call  IRClear
     movlw .255
     movwf irtimer
ReadTimer10
     bcf   flags,IRSilence  
     movfw irtimer
     subwf irvalue,w
     btfsc STATUS,C
     goto  ReadTimer11
     movfw irbit
     xorwf irtmp,F
ReadTimer11
     bcf   STATUS,C
     rrf   irbit,F
     movfw irbit
     btfsc STATUS,Z
     goto  ReadTimer12
     return
ReadTimer12
     movlw irbuf
     addwf ircnt,W
     movwf FSR
     movfw irtmp
     movwf INDF
     clrf  irtmp
     movfw irlength
     subwf ircnt,W
     btfsc STATUS,C
     return
     incf  ircnt,F    
     movlw b'10000000'
     movwf irbit
     movfw irlength
     subwf ircnt,W
     btfsc STATUS,C
     bsf   flags,IRAvail    
     return


IRReceived
     movfw ircnt
     subwf irlength,W
     btfss STATUS,Z
     goto  IRClear

     movlw .10
     movwf areg
     movlw .8
     movwf ereg
IRReceived1
     movfw irlength
     movwf breg
     movlw irbuf
     movwf FSR
IRReceived2
     movfw breg
     subwf irlength,W
     addwf ereg,W
     call  SetEpromAddr
     call  ReadEprom     
     movwf dreg
     movfw INDF
     subwf dreg,W
     btfss STATUS,Z
     goto  IRReceived3
     incf  FSR,F
     decfsz breg,F
     goto  IRReceived2

     call  GetStatus
     movwf asw
     movfw areg           ; up
     sublw .2
     btfss STATUS,Z
     goto  IRRecFound1
     incf  asw,F
     movfw asw
     sublw .7
     btfss STATUS,C
     clrf  asw
     call  SetStatus        
     goto  IRSendBuffer
IRRecFound1
     movfw areg           ; down
     sublw .1
     btfss STATUS,Z
     goto  IRRecFound2
     movlw .1
     subwf asw,F
     btfss STATUS,C
     goto  IRRecFound3
     call  SetStatus
     goto  IRSendBuffer
IRRecFound3
     movlw .7
     movwf asw
     call  SetStatus
     goto  IRSendBuffer

IRRecFound2
     movfw areg           ; number 1-8
     sublw .10
     movwf asw
     call  SetStatus   
     goto  IRSendBuffer

IRReceived3
     movlw .8
     addwf ereg,F     
     decfsz areg,F
     goto  IRReceived1


IRSendBuffer
     movfw ircnt
     movwf areg
     movlw irbuf        
     movwf FSR
IRSendBuffer1
     movfw INDF
     call  SendChar
     incf  FSR,F
     decfsz areg,F
     goto  IRSendBuffer1

IRClear
     clrf  ircnt
     movlw b'10000000'
     movwf irbit
     clrf  irtmp
     return


DelayBit1_0
     movlw delbit1
     movwf TMPD
N104 decfsz TMPD,F
     goto  N104
     return

DelayBit1_5
     movlw delbit2
     movwf TMPD
N156 decfsz TMPD,F
     goto  N156
     return

SendChar                  ; char in W
     bsf  STATUS,RP0
     bcf  TRISIO,TXLine
     bcf  STATUS,RP0

     movwf txchar
     movlw .8
     movwf txbit
     bcf INTCON,GIE
     bcf GPIO,TXLine
     call DelayBit1_0
SendChar0
     rrf txchar,F
     btfsc STATUS,C
     goto SendChar1
     bcf GPIO,TXLine
     goto SendChar2
SendChar1
     bsf GPIO,TXLine
SendChar2
     call DelayBit1_0
     decfsz txbit,F
     goto SendChar0
     bsf GPIO,TXLine

     bsf  STATUS,RP0
     bsf  TRISIO,TXLine
     bcf  STATUS,RP0

     bsf INTCON,GIE
     call DelayBit1_0
     call DelayBit1_0
     return


InitTimer1      
     bcf   T1CON,TMR1ON
     clrf  TMR1H
     clrf  TMR1L
     bcf   STATUS,RP0
     movlw b'00000100'
     movwf T1CON
     bsf   STATUS,RP0
     bsf   PIE1,TMR1IE
     bcf   STATUS,RP0
     bcf   PIR1,TMR1IF
     bsf   T1CON,TMR1ON
     return


Start
     bsf   STATUS,RP0
     call  0x3FF
     movwf OSCCAL
     bcf   STATUS,RP0
     clrf  tmr0
     clrf  flags
     clrf  rxbuf
     clrf  rxbit
     clrf GPIO
     movlw b'00000111'
     movwf CMCON
     bsf   STATUS,RP0
     movlw b'00001000'
     movwf IOC
     bcf   OPTION_REG,NOT_GPPU
     bcf   OPTION_REG,INTEDG
     clrf  ANSEL
     movlw b'11101100'
     movwf TRISIO           
     bcf   STATUS,RP0
     bsf   GPIO,TXLine
     bsf   GPIO,RXLine
     bsf   GPIO,IRinput
     call  InitTimer1
     movlw b'11011000'
     movwf INTCON         
     clrw
     call  ReadEprom
     movwf asw
     call  SetStatus

     call  IRClear
     movlw .1
     call  ReadEprom
     movwf irvalue
     movlw .4
     movwf irlength


Loop
     clrwdt
     call  CheckUSART
     call  CheckIR
     call  CheckButton
     goto  Loop


CheckIR
     btfsc flags,IRDetect
     goto  CheckIR1
     btfsc flags,IRAvail
     call  IRReceived
     bcf   flags,IRAvail
     return
CheckIR1
     movfw irtimer
     btfsc STATUS,Z
     return
     call  SendChar
     clrf  irtimer
     return

CheckButton
     btfsc GPIO,TXLine
     return
     call  GetStatus
     movwf asw
     incf  asw,F
     movfw asw
     sublw .7
     btfss STATUS,C
     clrf  asw
     call  SetStatus     
     call  WriteStatus     
     call  Delay200ms
     call  Delay200ms
     return

USARTReceive
;bsf GPIO,inputA
     btfsc GPIO,RXLine
     return
     movlw .8
     movwf rxbit
     clrf  rxchar
     call  DelayBit1_5
CheckUSARTReceive0
;bcf GPIO,inputA
     bcf   STATUS,C
     btfsc GPIO,RXLine
     bsf   STATUS,C
     rrf   rxchar,F
     call  DelayBit1_0
;bsf GPIO,inputA
     decfsz rxbit,F
     goto  CheckUSARTReceive0
     movlw rxlength
     subwf rxbuf,W
     btfsc STATUS,C
     return
     incf  rxbuf,F
     movlw rxbuf
     addwf rxbuf,W
     movwf FSR
     movfw rxchar
     movwf INDF
     return


CheckUSART
     movlw rxlength
     subwf rxbuf,W
     btfss STATUS,C
     return
     bcf   INTCON,GPIE
cmd0    
     cmp rxbuf+1,.0
     btfss STATUS,Z		
	 goto  cmd1
     call  SendBuffer
     goto  CheckUSARTEx
cmd1
     cmp rxbuf+1,.1
     btfss STATUS,Z		
	 goto  cmd2
     call  SendVersion
     goto  CheckUSARTEx
cmd2
     cmp rxbuf+1,.2
     btfss STATUS,Z		
	 goto  cmd3
     call  GetStatus
     call  SendChar
     goto  CheckUSARTEx
cmd3
     cmp rxbuf+1,.3
     btfss STATUS,Z		
	 goto  cmd4
     movfw rxbuf+2
     movwf asw
     call  SetStatus
     call  WriteStatus
     call  GetStatus 
     call  SendChar
     goto  CheckUSARTEx
cmd4
     cmp rxbuf+1,.4
     btfss STATUS,Z		
	 goto  cmd5
     bsf   flags,IRDetect
     goto  CheckUSARTEx
cmd5
     cmp rxbuf+1,.5
     btfss STATUS,Z		
	 goto  cmd6
     bcf   flags,IRDetect
     goto  CheckUSARTEx
cmd6
     cmp rxbuf+1,.6
     btfss STATUS,Z		
	 goto  cmd7
     movfw irvalue
     call  SendChar
     movlw .1
     call  SetEpromAddr
     movfw rxbuf+2
     movwf irvalue
     call  WriteEprom
     goto  CheckUSARTEx
cmd7
     cmp rxbuf+1,.7
     btfss STATUS,Z		
	 goto  cmd8
     movfw irvalue
     call  SendChar
     goto  CheckUSARTEx
cmd8
     cmp rxbuf+1,.8
     btfss STATUS,Z		
	 goto  cmd9

     goto  CheckUSARTEx
cmd9
     cmp rxbuf+1,.9
     btfss STATUS,Z		
	 goto  cmd10
     movfw rxbuf+2
     call  SetEpromAddr
     movfw rxbuf+3
     call  WriteEprom
     goto  CheckUSARTEx
cmd10
     cmp rxbuf+1,.10
     btfss STATUS,Z		
	 goto  cmd11
     movlw .80
     movwf areg
     movlw .8
     movwf dreg
sdcont
     movfw dreg
     call  SetEpromAddr
     incf  dreg,F
     call  ReadEprom
     call  SendChar
     decfsz areg,F
     goto  sdcont
     goto  CheckUSARTEx
cmd11

   
CheckUSARTEx
     call  SendCRLF
     clrf  rxbuf
     bsf   INTCON,GPIE
     return

GetStatus
     clrw
     btfsc GPIO,inputA
     addlw .1
     btfsc GPIO,inputB
     addlw .2
     btfsc GPIO,inputC
     addlw .4
     return

SetStatus
     btfss asw,0
     bcf   GPIO,inputA
     btfss asw,1
     bcf   GPIO,inputB
     btfss asw,2
     bcf   GPIO,inputC
     btfsc asw,0
     bsf   GPIO,inputA
     btfsc asw,1
     bsf   GPIO,inputB
     btfsc asw,2
     bsf   GPIO,inputC
     return

WriteStatus
     clrw
     call  SetEpromAddr
     movfw asw
     call  WriteEprom
     return

SendBuffer
     movfw rxbuf
     movwf CTMP
     movlw rxbuf
     movwf FSR
SendBuffer1
     incf  FSR,F
     movfw INDF
     call  SendChar
     decfsz CTMP,F
     goto  SendBuffer1
     movfw rxbuf
     call SendChar
     return

SendVersion
     clrf  CTMP
SendVersion1
     movlw HIGH version
     movwf PCLATH
     movfw CTMP
     call  version
     andlw 0xFF
     btfsc STATUS,Z
     return
     call  SendChar
     incf  CTMP,F
     goto  SendVersion1

SendCRLF
     movlw .13
     call  SendChar
     movlw .10
     call  SendChar
     return


;================================================================
; Delay uses DTMP1, DTMP2, DTMP3
;================================================================

Delay_
     movwf DTMP1
N0   movlw .100
     movwf DTMP2
N1   movlw .167
     movwf DTMP3
N2   decfsz DTMP3,F                
     goto  N2
     decfsz DTMP2,F
     goto  N1
     decfsz DTMP1,F                
     goto  N0
     return

Delay100ms
     movlw .2
     call  Delay_
     return

Delay200ms
     movlw .4
     call  Delay_
     return

;================================================================
; EEPROM
;================================================================

ReadEprom                 ; w = address to read
     bsf   STATUS, RP0
     movwf EEADR
     bsf   EECON1, RD
     movfw EEDATA
     bcf   STATUS, RP0
     return

SetEpromAddr              ; w = address to read
     bsf   STATUS, RP0
     movwf EEADR
     bcf   STATUS, RP0
     return

WriteEprom
     bsf   STATUS, RP0
     movwf EEDATA
     bsf   EECON1, WREN
     bcf   INTCON, GIE
     movlw 0x55
     movwf EECON2
     movlw 0xAA
     movwf EECON2
     bsf   EECON1,WR
     bsf   INTCON, GIE
     bcf   EECON1, WREN
     btfsc EECON1,WR
     goto  $-1
     bcf   STATUS, RP0
     bcf   PIR1,EEIF
     return



     ORG 0x3E0
version   
     addwf PCL,F
     retlw 'A'
     retlw 'u'
     retlw 'd'
     retlw 'i'
     retlw 'o'
     retlw ' '
     retlw 's'
     retlw 'w'
     retlw 'i'
     retlw 't'
     retlw 'c'
     retlw 'h'
     retlw ' '
     retlw 'v'
     retlw '1'
     retlw '.'
     retlw '0'
     retlw .0


 END

